#!/usr/bin/perl
#
# This code is distributed under the terms of the GPL
#
# IPCop UPS Server AddOn Script
#
# Script to read infos from apcupsd status and dump into rrd and generate some nice ups graphs
#
# $Id: upsgraphs.pl ver. 0.5.8 from 12/04/2008 s.feddersen Exp $
#
# Copyright (c) 2005-2008 by Stephan Feddersen
#

use strict;
use RRDs;

require "/var/ipcop/general-functions.pl";
require "${General::swroot}/lang.pl";

my (%settings, @ipacsum, $iface, $ERROR);

&General::readhash("${General::swroot}/ethernet/settings", \%settings);

use Encode 'from_to';
my %tr=();

if (${Lang::language} =~ /^(ar|bg|el|fa|gu|ja|ru|th|ur|vi|zh|zt)$/) {
	eval `/bin/cat "${General::swroot}/langs/en.pl"`;
} else {
	%tr=%Lang::tr;
}

my $rrdlog = "/var/log/rrd";
my $graphs = "/home/httpd/html/graphs";

$ENV{PATH}="/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin";

if ((${Lang::language} eq 'cs') || 
    (${Lang::language} eq 'hu') || 
    (${Lang::language} eq 'pl') || 
    (${Lang::language} eq 'sk')) {

	foreach my $key(keys %Lang::tr) {
		from_to($tr{$key}, "utf-8", "iso-8859-2");
	}
} elsif (${Lang::language} eq 'tr') {

	foreach my $key(keys %Lang::tr) {
		from_to($tr{$key}, "utf-8", "iso-8859-9");
	}
} else {
	foreach my $key(keys %Lang::tr) {
		from_to($tr{$key}, "utf-8", "iso-8859-1"); 
	}
}

sub updateupstempbatt1 {

	my $period = $_[0];

	RRDs::graph ("$graphs/ups_tempvolt_$period.png",
		"--start", "-1$period", "-aPNG", "-i", "-z",
		"--alt-autoscale",
		"--alt-y-grid", "-w 600", "-h 100",
		"--color", "SHADEA#EAE9EE",
		"--color", "SHADEB#EAE9EE",
		"--color", "BACK#EAE9EE",
		"-t $tr{'ups_graphs_tempvolt1'} ($tr{'graph per'} $tr{$period})",
		"-v $tr{'ups_graphs_voltcel'}",
		"DEF:BATTV=$rrdlog/ups.rrd:BATTV:AVERAGE",
		"DEF:TEMP=$rrdlog/ups.rrd:TEMP:AVERAGE",
		"LINE2:BATTV#3333FF:$tr{'ups_graphs_tempvolt2'}\\j",
		"LINE2:TEMP#FF3333:$tr{'ups_graphs_tempvolt3'}\\j",
		"GPRINT:TEMP:MAX:$tr{'ups_graphs_max'} $tr{'ups_graphs_tempvolt4'} \\:% 2.2lfC",
		"GPRINT:TEMP:AVERAGE:$tr{'ups_graphs_average'} $tr{'ups_graphs_tempvolt4'} \\:% 2.2lfC",
		"GPRINT:TEMP:LAST:$tr{'ups_graphs_current'} $tr{'ups_graphs_tempvolt4'} \\:% 2.2lfC\\j",
		"GPRINT:BATTV:MAX:$tr{'ups_graphs_max'} $tr{'ups_graphs_tempvolt5'} \\:% 2.2lf V",
		"GPRINT:BATTV:AVERAGE:$tr{'ups_graphs_average'} $tr{'ups_graphs_tempvolt5'} \\:% 2.2lf V",
		"GPRINT:BATTV:LAST:$tr{'ups_graphs_current'} $tr{'ups_graphs_tempvolt5'} \\:% 2.2lf V\\j");
	$ERROR = RRDs::error;
	print "Error in RRD::graph for upstempbatt: $ERROR\n" if $ERROR;
}

sub updateupstempbatt2 {

	my $period = $_[0];

	RRDs::graph ("$graphs/ups_tempvolt_$period.png",
		"--start", "-1$period", "-aPNG", "-i", "-z",
		"--alt-autoscale",
		"--alt-y-grid", "-w 600", "-h 100",
		"--color", "SHADEA#EAE9EE",
		"--color", "SHADEB#EAE9EE",
		"--color", "BACK#EAE9EE",
		"-t $tr{'ups_graphs_tempvolt12'} ($tr{'graph per'} $tr{$period})",
		"-v $tr{'ups_graphs_voltcel'}",
		"DEF:BATTV=$rrdlog/ups.rrd:BATTV:AVERAGE",
		"LINE2:BATTV#3333FF:$tr{'ups_graphs_tempvolt2'}\\j",
		"GPRINT:BATTV:MAX:$tr{'ups_graphs_max'} $tr{'ups_graphs_tempvolt5'} \\:% 2.2lf V",
		"GPRINT:BATTV:AVERAGE:$tr{'ups_graphs_average'} $tr{'ups_graphs_tempvolt5'} \\:% 2.2lf V",
		"GPRINT:BATTV:LAST:$tr{'ups_graphs_current'} $tr{'ups_graphs_tempvolt5'} \\:% 2.2lf V\\j");
	$ERROR = RRDs::error;
	print "Error in RRD::graph for upstempbatt: $ERROR\n" if $ERROR;
}

sub updateupstempbatt3 {

	my $period = $_[0];

	RRDs::graph ("$graphs/ups_tempvolt_$period.png",
		"--start", "-1$period", "-aPNG", "-i", "-z",
		"--alt-autoscale",
		"--alt-y-grid", "-w 600", "-h 100",
		"--color", "SHADEA#EAE9EE",
		"--color", "SHADEB#EAE9EE",
		"--color", "BACK#EAE9EE",
		"-t $tr{'ups_graphs_tempvolt13'} ($tr{'graph per'} $tr{$period})",
		"-v $tr{'ups_graphs_voltcel'}",
		"DEF:TEMP=$rrdlog/ups.rrd:TEMP:AVERAGE",
		"LINE2:TEMP#FF3333:$tr{'ups_graphs_tempvolt3'}\\j",
		"GPRINT:TEMP:MAX:$tr{'ups_graphs_max'} $tr{'ups_graphs_tempvolt4'} \\:% 2.2lfC",
		"GPRINT:TEMP:AVERAGE:$tr{'ups_graphs_average'} $tr{'ups_graphs_tempvolt4'} \\:% 2.2lfC",
		"GPRINT:TEMP:LAST:$tr{'ups_graphs_current'} $tr{'ups_graphs_tempvolt4'} \\:% 2.2lfC\\j");
	$ERROR = RRDs::error;
	print "Error in RRD::graph for upstempbatt: $ERROR\n" if $ERROR;
}

sub updatemaxminv {

	my $period = $_[0];
	
	RRDs::graph ("$graphs/ups_voltage_$period.png",
		"--start", "-1$period", "-aPNG", "-i", "-z", "-A",
		"--alt-autoscale",
		"--alt-y-grid", "-w 600", "-h 100",
		"--color", "SHADEA#EAE9EE",
		"--color", "SHADEB#EAE9EE",
		"--color", "BACK#EAE9EE",
		"-t $tr{'ups_graphs_maxminv'} ($tr{'graph per'} $tr{$period})",
		"-v $tr{'ups_graphs_volt'}",
		"DEF:MAXLINEV=$rrdlog/ups.rrd:MAXLINEV:AVERAGE",
		"DEF:MINLINEV=$rrdlog/ups.rrd:MINLINEV:AVERAGE",
		"LINE2:MAXLINEV#33FFFF:$tr{'ups_graphs_maxlinev'}\\j",
		"LINE2:MINLINEV#3333FF:$tr{'ups_graphs_minlinev'}\\j",
		"GPRINT:MAXLINEV:MAX:$tr{'ups_graphs_max'} $tr{'ups_graphs_maxlinev1'} \\:% 3.2lf V",
		"GPRINT:MAXLINEV:AVERAGE:$tr{'ups_graphs_average'} $tr{'ups_graphs_maxlinev'} \\:% 3.2lf V",
		"GPRINT:MAXLINEV:LAST:$tr{'ups_graphs_current'} $tr{'ups_graphs_maxlinev'} \\:% 3.2lf V\\j",
		"GPRINT:MINLINEV:MAX:$tr{'ups_graphs_minlinev'} \\:% 3.2lf V",
		"GPRINT:MINLINEV:AVERAGE:$tr{'ups_graphs_average'} $tr{'ups_graphs_minlinev'} \\:% 3.2lf V",
		"GPRINT:MINLINEV:LAST:$tr{'ups_graphs_current'} $tr{'ups_graphs_minlinev'} \\:% 3.2lf V\\j");
	$ERROR = RRDs::error;
	print "Error in RRD::graph for upstempbatt: $ERROR\n" if $ERROR;
}

sub updatetimeleft {

	my $period = $_[0];

	RRDs::graph ("$graphs/ups_timeleft_$period.png",
		"--start", "-1$period", "-aPNG", "-i", "-z",
		"--alt-autoscale",
		"--alt-y-grid", "-w 600", "-h 100",
		"--color", "SHADEA#EAE9EE",
		"--color", "SHADEB#EAE9EE",
		"--color", "BACK#EAE9EE",
		"-t $tr{'ups_graphs_timeleft1'} ($tr{'graph per'} $tr{$period})",
		"-v $tr{'ups_graphs_minute'}",
		"DEF:TIMELEFT=$rrdlog/ups.rrd:TIMELEFT:AVERAGE",
		"LINE2:TIMELEFT#339900:$tr{'ups_graphs_timeleft2'}\\j",
		"GPRINT:TIMELEFT:MAX:$tr{'ups_graphs_max'} $tr{'ups_graphs_timeleft3'} \\:% 3.2lf Min",
		"GPRINT:TIMELEFT:AVERAGE:$tr{'ups_graphs_average'} $tr{'ups_graphs_timeleft3'} \\:% 3.2lf Min",
		"GPRINT:TIMELEFT:LAST:$tr{'ups_graphs_current'} $tr{'ups_graphs_timeleft3'} \\:% 3.2lf Min\\j");
	$ERROR = RRDs::error;
	print "Error in RRD::graph for upstempbatt: $ERROR\n" if $ERROR;
}

sub updateoutputvoltage {

	my $period = $_[0];

	RRDs::graph ("$graphs/ups_outputvoltage_$period.png",
		"--start", "-1$period", "-aPNG", "-i", "-z",
		"--alt-autoscale",
		"--alt-y-grid", "-w 600", "-h 100",
		"--color", "SHADEA#EAE9EE",
		"--color", "SHADEB#EAE9EE",
		"--color", "BACK#EAE9EE",
		"-t $tr{'ups_graphs_outputvoltage1'} ($tr{'graph per'} $tr{$period})",
		"-v $tr{'ups_graphs_volt'}",
		"DEF:OUTPUTV=$rrdlog/ups.rrd:OUTPUTV:AVERAGE",
		"LINE2:OUTPUTV#FF3300:$tr{'ups_graphs_outputvoltage2'}\\j",
		"GPRINT:OUTPUTV:MAX:$tr{'ups_graphs_max'} $tr{'ups_graphs_outputvoltage1'} \\:% 3.2lf V",
		"GPRINT:OUTPUTV:AVERAGE:$tr{'ups_graphs_average'} $tr{'ups_graphs_outputvoltage1'} \\:% 3.2lf V",
		"GPRINT:OUTPUTV:LAST:$tr{'ups_graphs_current'} $tr{'ups_graphs_outputvoltage1'} \\:% 3.2lf V\\j");
	$ERROR = RRDs::error;
	print "Error in RRD::graph for upstempbatt: $ERROR\n" if $ERROR;
}

sub updateloadpct {

	my $period = $_[0];

	RRDs::graph ("$graphs/ups_loadpct_$period.png",
		"--start", "-1$period", "-aPNG", "-i", "-z",
		"--alt-autoscale",
		"--alt-y-grid", "-w 600", "-h 100",
		"--color", "SHADEA#EAE9EE",
		"--color", "SHADEB#EAE9EE",
		"--color", "BACK#EAE9EE",
		"-t $tr{'ups_graphs_loadpct1'} ($tr{'graph per'} $tr{$period})",
		"-v $tr{'ups_graphs_percent'}",
		"DEF:LOADPCT=$rrdlog/ups.rrd:LOADPCT:AVERAGE",
		"LINE2:LOADPCT#00FF00:$tr{'ups_graphs_loadpct2'}\\j",
		"GPRINT:LOADPCT:MAX:$tr{'ups_graphs_max'} $tr{'ups_graphs_loadpct1'} \\:% 3.2lf %%",
		"GPRINT:LOADPCT:AVERAGE:$tr{'ups_graphs_average'} $tr{'ups_graphs_loadpct1'} \\:% 3.2lf %%",
		"GPRINT:LOADPCT:LAST:$tr{'ups_graphs_current'} $tr{'ups_graphs_loadpct1'} \\:% 3.2lf %%\\j");
	$ERROR = RRDs::error;
	print "Error in RRD::graph for upstempbatt: $ERROR\n" if $ERROR;
}

sub updatemaxmincharge {

	my $period = $_[0];

	RRDs::graph ("$graphs/ups_charge_$period.png",
		"--start", "-1$period", "-aPNG", "-i", "-z",
		"--alt-autoscale",
		"--alt-y-grid", "-w 600", "-h 100",
		"--color", "SHADEA#EAE9EE",
		"--color", "SHADEB#EAE9EE",
		"--color", "BACK#EAE9EE",
		"-t $tr{'ups_graphs_charge1'} ($tr{'graph per'} $tr{$period})",
		"-v $tr{'ups_graphs_percent'}",
		"DEF:MBATTCHG=$rrdlog/ups.rrd:MBATTCHG:AVERAGE",
		"DEF:BCHARGE=$rrdlog/ups.rrd:BCHARGE:AVERAGE",
		"LINE2:MBATTCHG#3333FF:$tr{'ups_graphs_charge3'}\\j",
		"LINE2:BCHARGE#FF3333:$tr{'ups_graphs_charge2'}\\j",
		"GPRINT:MBATTCHG:MAX:$tr{'ups_graphs_charge3'} \\:% 3.2lf %%",
		"GPRINT:MBATTCHG:AVERAGE:$tr{'ups_graphs_average'} $tr{'ups_graphs_charge3'} \\:% 3.2lf %%",
		"GPRINT:MBATTCHG:LAST:$tr{'ups_graphs_current'} $tr{'ups_graphs_charge3'} \\:% 3.2lf %%\\j",
		"GPRINT:BCHARGE:MAX:$tr{'ups_graphs_charge2'} \\:% 3.2lf %%",
		"GPRINT:BCHARGE:AVERAGE:$tr{'ups_graphs_average'} $tr{'ups_graphs_charge2'} \\:% 3.2lf %%",
		"GPRINT:BCHARGE:LAST:$tr{'ups_graphs_current'} $tr{'ups_graphs_charge2'} \\:% 3.2lf %%\\j");
	$ERROR = RRDs::error;
	print "Error in RRD::graph for upstempbatt: $ERROR\n" if $ERROR;
}

sub updateupsrrd
{

if (! -e "$rrdlog/ups.rrd")

	{
		RRDs::create ("$rrdlog/ups.rrd", "--step=300",
			"DS:TEMP:GAUGE:600:U:U",
			"DS:BATTV:GAUGE:600:U:U",
			"DS:MAXLINEV:GAUGE:600:U:U",
			"DS:MINLINEV:GAUGE:600:U:U",
			"DS:LOTRANS:GAUGE:600:U:U",
			"DS:HITRANS:GAUGE:600:U:U",
			"DS:TIMELEFT:GAUGE:600:U:U",
			"DS:LINEV:GAUGE:600:U:U",
			"DS:LOADPCT:GAUGE:600:U:U",
			"DS:BCHARGE:GAUGE:600:U:U",
			"DS:MBATTCHG:GAUGE:600:U:U",
			"DS:MINTIMEL:GAUGE:600:U:U",
			"DS:MAXTIME:GAUGE:600:U:U",
			"DS:OUTPUTV:GAUGE:600:U:U",
			"RRA:AVERAGE:0.5:1:576",
			"RRA:AVERAGE:0.5:6:672",
			"RRA:AVERAGE:0.5:24:732",
			"RRA:AVERAGE:0.5:144:1460");
		$ERROR = RRDs::error;
		print "Error in RRD::create for ups: $ERROR\n" if $ERROR;
	}

}

sub updateupsdata
{

	my $data = "/var/log/ups/status";

	my $TEMP=`/bin/cat $data | grep -w ITEMP | awk '{print \$3 }'`;
	chomp ($TEMP);
	my $BATTV=`/bin/cat $data | grep -w BATTV | awk '{print \$3 }'`;
	chomp ($BATTV);
	my $MAXLINEV=`/bin/cat $data | grep -w MAXLINEV | awk '{print \$3 }'`;
	chomp ($MAXLINEV);
	my $MINLINEV=`/bin/cat $data | grep -w MINLINEV | awk '{print \$3 }'`;
	chomp ($MINLINEV);
	my $LOTRANS=`/bin/cat $data | grep -w LOTRANS | awk '{print \$3 }'`;
	chomp ($LOTRANS);
	my $HITRANS=`/bin/cat $data | grep -w HITRANS | awk '{print \$3 }'`;
	chomp ($HITRANS);
	my $TIMELEFT=`/bin/cat $data | grep -w TIMELEFT | awk '{print \$3 }'`;
	chomp ($TIMELEFT);
	my $LINEV=`/bin/cat $data | grep -w LINEV | awk '{print \$3 }'`;
	chomp ($LINEV);
	my $LOADPCT=`/bin/cat $data | grep -w LOADPCT | awk '{print \$3 }'`;
	chomp ($LOADPCT);
	my $BCHARGE=`/bin/cat $data | grep -w BCHARGE | awk '{print \$3 }'`;
	chomp ($BCHARGE);
	my $MBATTCHG=`/bin/cat $data | grep -w MBATTCHG | awk '{print \$3 }'`;
	chomp ($MBATTCHG);
	my $MINTIMEL=`/bin/cat $data | grep -w MINTIMEL | awk '{print \$3 }'`;
	chomp ($MINTIMEL);
	my $MAXTIME=`/bin/cat $data | grep -w MAXTIME | awk '{print \$3 }'`;
	chomp ($MAXTIME);
	my $OUTPUTV=`/bin/cat $data | grep -w OUTPUTV | awk '{print \$3 }'`;
	chomp ($OUTPUTV);

	RRDs::update ("$rrdlog/ups.rrd",
		"N:$TEMP:$BATTV:$MAXLINEV:$MINLINEV:$LOTRANS:$HITRANS:$TIMELEFT:$LINEV:$LOADPCT:$BCHARGE:$MBATTCHG:$MINTIMEL:$MAXTIME:$OUTPUTV");
	$ERROR = RRDs::error;
	print "Error in RRD::update for UPS: $ERROR\n" if $ERROR;

}

updateupsrrd ();

my $pidalive = 0;

my %upssettings =();

&General::readhash("${General::swroot}/apcupsd/settings", \%upssettings);

my %servicenames =
(
$Lang::tr{'ups'} => 'apcupsd',
);

isrunning ('apcupsd');

if ($pidalive)
{

updateupsdata ();

if ($upssettings{'UPSTYPE'} eq 'apcsmart' || $upssettings{'UPSTYPE'} eq 'usb')
{
	updateupstempbatt1 ("day");
	updateupstempbatt1 ("week");
	updateupstempbatt1 ("month");
	updateupstempbatt1 ("year");
}

if ($upssettings{'UPSTYPE'} eq 'dumb')
{
	updateupstempbatt2 ("day");
	updateupstempbatt2 ("week");
	updateupstempbatt2 ("month");
	updateupstempbatt2 ("year");
}

if ($upssettings{'UPSTYPE'} eq 'snmp')
{
	updateupstempbatt3 ("day");
	updateupstempbatt3 ("week");
	updateupstempbatt3 ("month");
	updateupstempbatt3 ("year");
}

updatemaxminv ("day");
updatemaxminv ("week");
updatemaxminv ("month");
updatemaxminv ("year");
updatetimeleft ("day");
updatetimeleft ("week");
updatetimeleft ("month");
updatetimeleft ("year");
updateoutputvoltage ("day");
updateoutputvoltage ("week");
updateoutputvoltage ("month");
updateoutputvoltage ("year");
updateloadpct ("day");
updateloadpct ("week");
updateloadpct ("month");
updateloadpct ("year");
updatemaxmincharge ("day");
updatemaxmincharge ("week");
updatemaxmincharge ("month");
updatemaxmincharge ("year");

}

sub isrunning
{
	my $cmd = $_[0];
	my $status = "<td align='center' bgcolor='${Header::colourred}'><font color='white'><b>$Lang::tr{'stopped'}</b></font></td>";
	my $pid = '';
	my $testcmd = '';
	my $exename;

	$pidalive = 0;

	$cmd =~ /(^[a-z]+)/;
	$exename = $1;

	if (open(FILE, "/var/run/${cmd}.pid"))
	{
 		$pid = <FILE>; chomp $pid;
		close FILE;
		if (open(FILE, "/proc/${pid}/status"))
		{
			while (<FILE>)
			{
				if (/^Name:\W+(.*)/) {
					$testcmd = $1; }
			}
			close FILE;
			if ($testcmd =~ /$exename/)
			{
				$status = "<td align='center' bgcolor='${Header::colourgreen}'><font color='white'><b>$Lang::tr{'running'}</b></font></td>";
				$pidalive = 1;
			}
		}
	}

	return $status;
}
